@@ -30,31 +30,33 @@
  * Modified by the GTK+ Team and others 2007.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#include <config.h>
+#include "config.h"
 
+#ifndef _WIN32
 #define _GNU_SOURCE
+#endif
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
+
 #include "gtkmain.h"
 #include "gtkintl.h"
-#include "gtkrange.h"
 #include "gtkbindings.h"
-#include "gtkscale.h"
+#include "gtkhscale.h"
 #include "gtkvscale.h"
 #include "gtkframe.h"
+#include "gtkhbox.h"
 #include "gtkvbox.h"
 #include "gtkwindow.h"
 #include "gtkmarshalers.h"
 #include "gtkstock.h"
 #include "gtkprivate.h"
-
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk/gdkkeysyms.h>
 #include "gtkscalebutton.h"
 
 #include "gtkalias.h"
@@ -74,7 +76,8 @@ enum
 enum
 {
   PROP_0,
-  
+
+  PROP_ORIENTATION,
   PROP_VALUE,
   PROP_SIZE,
   PROP_ADJUSTMENT,
@@ -90,18 +93,22 @@ struct _GtkScaleButtonPrivate
   GtkWidget *image;
 
   GtkIconSize size;
-  
+  GtkOrientation orientation;
+
   guint click_id;
   gint click_timeout;
   guint timeout : 1;
   gdouble direction;
   guint32 pop_time;
-  
+
   gchar **icon_list;
+
+  GtkAdjustment *adjustment; /* needed because it must be settable in init() */
 };
 
-static void	gtk_scale_button_class_init	(GtkScaleButtonClass *klass);
-static void	gtk_scale_button_init		(GtkScaleButton      *button);
+static GObject* gtk_scale_button_constructor    (GType                  type,
+                                                 guint                  n_construct_properties,
+                                                 GObjectConstructParam *construct_params);
 static void	gtk_scale_button_dispose	(GObject             *object);
 static void     gtk_scale_button_finalize       (GObject             *object);
 static void	gtk_scale_button_set_property	(GObject             *object,
@@ -120,8 +127,8 @@ static gboolean	gtk_scale_button_press		(GtkWidget           *widget,
 						 GdkEventButton      *event);
 static gboolean gtk_scale_button_key_release	(GtkWidget           *widget,
     						 GdkEventKey         *event);
-static void gtk_scale_button_popup_from_bindings (GtkWidget *widget);
-static void gtk_scale_button_popdown_from_bindings (GtkWidget *widget);
+static void     gtk_scale_button_popup          (GtkWidget           *widget);
+static void     gtk_scale_button_popdown        (GtkWidget           *widget);
 static gboolean cb_dock_button_press		(GtkWidget           *widget,
 						 GdkEventButton      *event,
 						 gpointer             user_data);
@@ -147,10 +154,7 @@ static void gtk_scale_button_update_icon	(GtkScaleButton      *button);
 static void gtk_scale_button_scale_value_changed(GtkRange            *range);
 
 /* see below for scale definitions */
-static GtkWidget *gtk_scale_button_scale_new	(GtkScaleButton      *button,
-						 gdouble              min,
-						 gdouble              max,
-						 gdouble              step);
+static GtkWidget *gtk_scale_button_scale_box_new(GtkScaleButton      *button);
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
@@ -165,16 +169,33 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
 
   g_type_class_add_private (klass, sizeof (GtkScaleButtonPrivate));
 
+  gobject_class->constructor = gtk_scale_button_constructor;
   gobject_class->finalize = gtk_scale_button_finalize;
   gobject_class->dispose = gtk_scale_button_dispose;
   gobject_class->set_property = gtk_scale_button_set_property;
   gobject_class->get_property = gtk_scale_button_get_property;
-  
+
   widget_class->button_press_event = gtk_scale_button_press;
   widget_class->key_release_event = gtk_scale_button_key_release;
   widget_class->scroll_event = gtk_scale_button_scroll;
   widget_class->screen_changed = gtk_scale_button_screen_changed;
 
+  /**
+   * GtkScaleButton:orientation:
+   *
+   * The orientation of the #GtkScaleButton's popup window.
+   *
+   * Since: 2.14
+   **/
+  g_object_class_install_property (gobject_class,
+				   PROP_ORIENTATION,
+				   g_param_spec_enum ("orientation",
+                                                      P_("Orientation"),
+                                                      P_("The orientation of the scale"),
+                                                      GTK_TYPE_ORIENTATION,
+                                                      GTK_ORIENTATION_VERTICAL,
+                                                      GTK_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class,
 				   PROP_VALUE,
 				   g_param_spec_double ("value",
@@ -205,20 +226,20 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
   /**
    * GtkScaleButton:icons:
    *
-   * The names of the icons to be used by the scale button. 
-   * The first item in the array will be used in the button 
-   * when the current value is the lowest value, the second 
-   * item for the highest value. All the subsequent icons will 
-   * be used for all the other values, spread evenly over the 
+   * The names of the icons to be used by the scale button.
+   * The first item in the array will be used in the button
+   * when the current value is the lowest value, the second
+   * item for the highest value. All the subsequent icons will
+   * be used for all the other values, spread evenly over the
    * range of values.
    *
-   * If there's only one icon name in the @icons array, it will 
-   * be used for all the values. If only two icon names are in 
-   * the @icons array, the first one will be used for the bottom 
+   * If there's only one icon name in the @icons array, it will
+   * be used for all the values. If only two icon names are in
+   * the @icons array, the first one will be used for the bottom
    * 50% of the scale, and the second one for the top 50%.
    *
-   * It is recommended to use at least 3 icons so that the 
-   * #GtkScaleButton reflects the current value of the scale 
+   * It is recommended to use at least 3 icons so that the
+   * #GtkScaleButton reflects the current value of the scale
    * better for the users.
    *
    * Since: 2.12
@@ -249,13 +270,13 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
 		  NULL, NULL,
 		  _gtk_marshal_VOID__DOUBLE,
 		  G_TYPE_NONE, 1, G_TYPE_DOUBLE);
-  
+
   /**
    * GtkScaleButton::popup:
    * @button: the object which received the signal
    *
-   * The ::popup signal is a 
-   * <link linkend="keybinding-signals">keybinding signal</link> 
+   * The ::popup signal is a
+   * <link linkend="keybinding-signals">keybinding signal</link>
    * which gets emitted to popup the scale widget.
    *
    * The default bindings for this signal are Space, Enter and Return.
@@ -263,20 +284,20 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
    * Since: 2.12
    */
   signals[POPUP] =
-    _gtk_binding_signal_new (I_("popup"),
-			     G_OBJECT_CLASS_TYPE (klass),
-			     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-			     G_CALLBACK (gtk_scale_button_popup_from_bindings),
-			     NULL, NULL,
-			     g_cclosure_marshal_VOID__VOID,
-			     G_TYPE_NONE, 0);
+    g_signal_new_class_handler (I_("popup"),
+                                G_OBJECT_CLASS_TYPE (klass),
+                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                G_CALLBACK (gtk_scale_button_popup),
+                                NULL, NULL,
+                                g_cclosure_marshal_VOID__VOID,
+                                G_TYPE_NONE, 0);
 
   /**
    * GtkScaleButton::popdown:
    * @button: the object which received the signal
    *
-   * The ::popdown signal is a 
-   * <link linkend="keybinding-signals">keybinding signal</link> 
+   * The ::popdown signal is a
+   * <link linkend="keybinding-signals">keybinding signal</link>
    * which gets emitted to popdown the scale widget.
    *
    * The default binding for this signal is Escape.
@@ -284,19 +305,21 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
    * Since: 2.12
    */
   signals[POPDOWN] =
-    _gtk_binding_signal_new (I_("popdown"),
-			     G_OBJECT_CLASS_TYPE (klass),
-			     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-			     G_CALLBACK (gtk_scale_button_popdown_from_bindings),
-			     NULL, NULL,
-			     g_cclosure_marshal_VOID__VOID,
-			     G_TYPE_NONE, 0);
+    g_signal_new_class_handler (I_("popdown"),
+                                G_OBJECT_CLASS_TYPE (klass),
+                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                G_CALLBACK (gtk_scale_button_popdown),
+                                NULL, NULL,
+                                g_cclosure_marshal_VOID__VOID,
+                                G_TYPE_NONE, 0);
 
   /* Key bindings */
   binding_set = gtk_binding_set_by_class (widget_class);
 
   gtk_binding_entry_add_signal (binding_set, GDK_space, 0,
 				"popup", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_KP_Space, 0,
+				"popup", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_Return, 0,
 				"popup", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_ISO_Enter, 0,
@@ -310,7 +333,6 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
 static void
 gtk_scale_button_init (GtkScaleButton *button)
 {
-  GtkWidget *frame, *box;
   GtkScaleButtonPrivate *priv;
 
   button->priv = priv = GET_PRIVATE (button);
@@ -318,6 +340,7 @@ gtk_scale_button_init (GtkScaleButton *button)
   priv->timeout = FALSE;
   priv->click_id = 0;
   priv->click_timeout = CLICK_TIMEOUT;
+  priv->orientation = GTK_ORIENTATION_VERTICAL;
 
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
@@ -329,6 +352,7 @@ gtk_scale_button_init (GtkScaleButton *button)
 
   /* window */
   priv->dock = gtk_window_new (GTK_WINDOW_POPUP);
+  gtk_widget_set_name (priv->dock, "gtk-scalebutton-popup-window");
   g_signal_connect (priv->dock, "button-press-event",
 		    G_CALLBACK (cb_dock_button_press), button);
   g_signal_connect (priv->dock, "key-release-event",
@@ -339,13 +363,6 @@ gtk_scale_button_init (GtkScaleButton *button)
 		    G_CALLBACK (cb_dock_grab_broken_event), button);
   gtk_window_set_decorated (GTK_WINDOW (priv->dock), FALSE);
 
-  /* frame */
-  frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
-  gtk_container_add (GTK_CONTAINER (priv->dock), frame);
-  box = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (frame), box);
-
   /* + */
   button->plus_button = gtk_button_new_with_label ("+");
   gtk_button_set_relief (GTK_BUTTON (button->plus_button), GTK_RELIEF_NONE);
@@ -353,16 +370,6 @@ gtk_scale_button_init (GtkScaleButton *button)
 		    G_CALLBACK (cb_button_press), button);
   g_signal_connect (button->plus_button, "button-release-event",
 		    G_CALLBACK (cb_button_release), button);
-  gtk_box_pack_start (GTK_BOX (box), button->plus_button, TRUE, FALSE, 0);
-
-  /* scale */
-  priv->scale = gtk_scale_button_scale_new (button, 0., 100., 2.);
-  gtk_widget_set_size_request (priv->scale, -1, SCALE_SIZE);
-  gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
-  gtk_range_set_inverted (GTK_RANGE (priv->scale), TRUE);
-  gtk_box_pack_start (GTK_BOX (box), priv->scale, TRUE, FALSE, 0);
-  g_signal_connect (priv->scale, "grab-notify",
-		    G_CALLBACK (cb_scale_grab_notify), button);
 
   /* - */
   button->minus_button = gtk_button_new_with_label ("-");
@@ -371,11 +378,41 @@ gtk_scale_button_init (GtkScaleButton *button)
 		   G_CALLBACK (cb_button_press), button);
   g_signal_connect (button->minus_button, "button-release-event",
 		    G_CALLBACK (cb_button_release), button);
-  gtk_box_pack_start (GTK_BOX (box), button->minus_button, TRUE, FALSE, 0);
+
+  priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 100.0, 2, 20, 0));
+  g_object_ref_sink (priv->adjustment);
+}
+
+static GObject *
+gtk_scale_button_constructor (GType                  type,
+                              guint                  n_construct_properties,
+                              GObjectConstructParam *construct_params)
+{
+  GObject *object;
+  GtkScaleButton *button;
+  GtkWidget *frame, *box;
+  GtkScaleButtonPrivate *priv;
+
+  object = G_OBJECT_CLASS (gtk_scale_button_parent_class)->constructor (type, n_construct_properties, construct_params);
+
+  button = GTK_SCALE_BUTTON (object);
+
+  priv = button->priv;
+
+  /* frame */
+  frame = gtk_frame_new (NULL);
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+  gtk_container_add (GTK_CONTAINER (priv->dock), frame);
+
+  /* box with scale and +/- buttons */
+  box = gtk_scale_button_scale_box_new (button);
+  gtk_container_add (GTK_CONTAINER (frame), box);
 
   /* set button text and size */
   priv->size = GTK_ICON_SIZE_SMALL_TOOLBAR;
   gtk_scale_button_update_icon (button);
+
+  return object;
 }
 
 static void
@@ -384,12 +421,13 @@ gtk_scale_button_set_property (GObject       *object,
 			       const GValue  *value,
 			       GParamSpec    *pspec)
 {
-  GtkScaleButton *button;
-
-  button = GTK_SCALE_BUTTON (object);
+  GtkScaleButton *button = GTK_SCALE_BUTTON (object);
 
   switch (prop_id)
     {
+    case PROP_ORIENTATION:
+      gtk_scale_button_set_orientation (button, g_value_get_enum (value));
+      break;
     case PROP_VALUE:
       gtk_scale_button_set_value (button, g_value_get_double (value));
       break;
@@ -408,7 +446,7 @@ gtk_scale_button_set_property (GObject       *object,
       gtk_scale_button_set_adjustment (button, g_value_get_object (value));
       break;
     case PROP_ICONS:
-      gtk_scale_button_set_icons (button, 
+      gtk_scale_button_set_icons (button,
                                   (const gchar **)g_value_get_boxed (value));
       break;
     default:
@@ -417,20 +455,20 @@ gtk_scale_button_set_property (GObject       *object,
     }
 }
 
-static void 
+static void
 gtk_scale_button_get_property (GObject     *object,
 			       guint        prop_id,
 			       GValue      *value,
 			       GParamSpec  *pspec)
 {
-  GtkScaleButton *button;
-  GtkScaleButtonPrivate *priv;
-
-  button = GTK_SCALE_BUTTON (object);
-  priv = button->priv;
+  GtkScaleButton *button = GTK_SCALE_BUTTON (object);
+  GtkScaleButtonPrivate *priv = button->priv;
 
   switch (prop_id)
     {
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, priv->orientation);
+      break;
     case PROP_VALUE:
       g_value_set_double (value, gtk_scale_button_get_value (button));
       break;
@@ -454,13 +492,19 @@ gtk_scale_button_finalize (GObject *object)
 {
   GtkScaleButton *button = GTK_SCALE_BUTTON (object);
   GtkScaleButtonPrivate *priv = button->priv;
-  
+
   if (priv->icon_list)
     {
       g_strfreev (priv->icon_list);
       priv->icon_list = NULL;
     }
-  
+
+  if (priv->adjustment)
+    {
+      g_object_unref (priv->adjustment);
+      priv->adjustment = NULL;
+    }
+
   G_OBJECT_CLASS (gtk_scale_button_parent_class)->finalize (object);
 }
 
@@ -486,7 +530,7 @@ gtk_scale_button_dispose (GObject *object)
 }
 
 /**
- * gtk_scale_button_new
+ * gtk_scale_button_new:
  * @size: a stock icon size
  * @min: the minimum value of the scale (usually 0)
  * @max: the maximum value of the scale (usually 100)
@@ -513,8 +557,8 @@ gtk_scale_button_new (GtkIconSize   size,
   GtkObject *adj;
 
   adj = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
-  
-  button = g_object_new (GTK_TYPE_SCALE_BUTTON, 
+
+  button = g_object_new (GTK_TYPE_SCALE_BUTTON,
                          "adjustment", adj,
                          "icons", icons,
                          "size", size,
@@ -524,7 +568,7 @@ gtk_scale_button_new (GtkIconSize   size,
 }
 
 /**
- * gtk_scale_button_get_value
+ * gtk_scale_button_get_value:
  * @button: a #GtkScaleButton
  *
  * Gets the current value of the scale button.
@@ -542,17 +586,17 @@ gtk_scale_button_get_value (GtkScaleButton * button)
 
   priv = button->priv;
 
-  return gtk_range_get_value (GTK_RANGE (priv->scale));
+  return gtk_adjustment_get_value (priv->adjustment);
 }
 
 /**
- * gtk_scale_button_set_value
+ * gtk_scale_button_set_value:
  * @button: a #GtkScaleButton
  * @value: new value of the scale button
  *
- * Sets the current value of the scale; if the value is outside 
- * the minimum or maximum range values, it will be clamped to fit 
- * inside them. The scale button emits the #GtkScaleButton::value-changed 
+ * Sets the current value of the scale; if the value is outside
+ * the minimum or maximum range values, it will be clamped to fit
+ * inside them. The scale button emits the #GtkScaleButton::value-changed
  * signal if the value changes.
  *
  * Since: 2.12
@@ -571,11 +615,11 @@ gtk_scale_button_set_value (GtkScaleButton *button,
 }
 
 /**
- * gtk_scale_button_set_icons
+ * gtk_scale_button_set_icons:
  * @button: a #GtkScaleButton
  * @icons: a %NULL-terminated array of icon names
  *
- * Sets the icons to be used by the scale button. 
+ * Sets the icons to be used by the scale button.
  * For details, see the #GtkScaleButton:icons property.
  *
  * Since: 2.12
@@ -613,22 +657,17 @@ gtk_scale_button_set_icons (GtkScaleButton  *button,
 GtkAdjustment*
 gtk_scale_button_get_adjustment	(GtkScaleButton *button)
 {
-  GtkRange *range;
-
   g_return_val_if_fail (GTK_IS_SCALE_BUTTON (button), NULL);
 
-  range = GTK_RANGE (button->priv->scale);
-  g_return_val_if_fail (range != NULL, NULL);
-
-  return gtk_range_get_adjustment (range);
+  return button->priv->adjustment;
 }
 
 /**
- * gtk_scale_button_set_adjustment
+ * gtk_scale_button_set_adjustment:
  * @button: a #GtkScaleButton
  * @adjustment: a #GtkAdjustment
  *
- * Sets the #GtkAdjustment to be used as a model 
+ * Sets the #GtkAdjustment to be used as a model
  * for the #GtkScaleButton's scale.
  * See gtk_range_set_adjustment() for details.
  *
@@ -639,12 +678,146 @@ gtk_scale_button_set_adjustment	(GtkScaleButton *button,
 				 GtkAdjustment  *adjustment)
 {
   g_return_if_fail (GTK_IS_SCALE_BUTTON (button));
-  g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  if (!adjustment)
+    adjustment = (GtkAdjustment*) gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+  else
+    g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
-  gtk_range_set_adjustment (GTK_RANGE (button->priv->scale),
-			    adjustment);
+  if (button->priv->adjustment != adjustment)
+    {
+      if (button->priv->adjustment)
+        g_object_unref (button->priv->adjustment);
+      button->priv->adjustment = g_object_ref_sink (adjustment);
 
-  g_object_notify (G_OBJECT (button), "adjustment");
+      if (button->priv->scale)
+        gtk_range_set_adjustment (GTK_RANGE (button->priv->scale), adjustment);
+
+      g_object_notify (G_OBJECT (button), "adjustment");
+    }
+}
+
+/**
+ * gtk_scale_button_get_orientation:
+ * @button: a #GtkScaleButton
+ *
+ * Gets the orientation of the #GtkScaleButton's popup window.
+ *
+ * Returns: the #GtkScaleButton's orientation.
+ *
+ * Since: 2.14
+ **/
+GtkOrientation
+gtk_scale_button_get_orientation (GtkScaleButton *button)
+{
+  g_return_val_if_fail (GTK_IS_SCALE_BUTTON (button), GTK_ORIENTATION_VERTICAL);
+
+  return button->priv->orientation;
+}
+
+/**
+ * gtk_scale_button_set_orientation:
+ * @button: a #GtkScaleButton
+ * @orientation: the new orientation
+ *
+ * Sets the orientation of the #GtkScaleButton's popup window.
+ *
+ * Since: 2.14
+ **/
+void
+gtk_scale_button_set_orientation (GtkScaleButton *button,
+                                  GtkOrientation  orientation)
+{
+  GtkScaleButtonPrivate *priv;
+
+  g_return_if_fail (GTK_IS_SCALE_BUTTON (button));
+
+  priv = button->priv;
+
+  if (orientation != priv->orientation)
+    {
+      priv->orientation = orientation;
+
+      if (priv->scale)
+        {
+          GtkWidget *box = priv->scale->parent;
+          GtkWidget *frame = box->parent;
+
+          g_object_ref (button->plus_button);
+          g_object_ref (button->minus_button);
+
+          gtk_container_remove (GTK_CONTAINER (box), button->plus_button);
+          gtk_container_remove (GTK_CONTAINER (box), button->minus_button);
+          gtk_container_remove (GTK_CONTAINER (box), priv->scale);
+          gtk_container_remove (GTK_CONTAINER (frame), box);
+
+          box = gtk_scale_button_scale_box_new (button);
+          gtk_container_add (GTK_CONTAINER (frame), box);
+
+          /* FIXME: without this, the popup window appears as a square
+           * after changing the orientation
+           */
+          gtk_window_resize (GTK_WINDOW (priv->dock), 1, 1);
+
+          g_object_unref (button->plus_button);
+          g_object_unref (button->minus_button);
+        }
+
+      g_object_notify (G_OBJECT (button), "orientation");
+    }
+}
+
+/**
+ * gtk_scale_button_get_plus_button:
+ * @button: a #GtkScaleButton
+ *
+ * Retrieves the plus button of the #GtkScaleButton.
+ *
+ * Returns: the plus button of the #GtkScaleButton.
+ *
+ * Since: 2.14
+ */
+GtkWidget *
+gtk_scale_button_get_plus_button (GtkScaleButton *button)
+{
+  g_return_val_if_fail (GTK_IS_SCALE_BUTTON (button), NULL);
+
+  return button->plus_button;
+}
+
+/**
+ * gtk_scale_button_get_minus_button:
+ * @button: a #GtkScaleButton
+ *
+ * Retrieves the minus button of the #GtkScaleButton.
+ *
+ * Returns: the minus button of the #GtkScaleButton.
+ *
+ * Since: 2.14
+ */
+GtkWidget *
+gtk_scale_button_get_minus_button (GtkScaleButton *button)
+{
+  g_return_val_if_fail (GTK_IS_SCALE_BUTTON (button), NULL);
+
+  return button->minus_button;
+}
+
+/**
+ * gtk_scale_button_get_popup:
+ * @button: a #GtkScaleButton
+ *
+ * Retrieves the popup of the #GtkScaleButton.
+ *
+ * Returns: the popup of the #GtkScaleButton
+ *
+ * Since: 2.14
+ */
+GtkWidget *
+gtk_scale_button_get_popup (GtkScaleButton *button)
+{
+  g_return_val_if_fail (GTK_IS_SCALE_BUTTON (button), NULL);
+
+  return button->priv->dock;
 }
 
 /*
@@ -662,7 +835,7 @@ gtk_scale_button_scroll (GtkWidget      *widget,
 
   button = GTK_SCALE_BUTTON (widget);
   priv = button->priv;
-  adj = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  adj = priv->adjustment;
 
   if (event->type != GDK_SCROLL)
     return FALSE;
@@ -720,14 +893,14 @@ gtk_scale_popup (GtkWidget *widget,
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
   GtkAdjustment *adj;
-  gint x, y, m, dx, dy, sx, sy, ystartoff;
+  gint x, y, m, dx, dy, sx, sy, startoff;
   gdouble v;
   GdkDisplay *display;
   GdkScreen *screen;
 
   button = GTK_SCALE_BUTTON (widget);
   priv = button->priv;
-  adj = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  adj = priv->adjustment;
 
   display = gtk_widget_get_display (widget);
   screen = gtk_widget_get_screen (widget);
@@ -738,23 +911,49 @@ gtk_scale_popup (GtkWidget *widget,
   gdk_window_get_origin (widget->window, &x, &y);
   x += widget->allocation.x;
   y += widget->allocation.y;
-  gtk_window_move (GTK_WINDOW (priv->dock), x, y - (SCALE_SIZE / 2));
+
+  if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+    gtk_window_move (GTK_WINDOW (priv->dock), x, y - (SCALE_SIZE / 2));
+  else
+    gtk_window_move (GTK_WINDOW (priv->dock), x - (SCALE_SIZE / 2), y);
+
   gtk_widget_show_all (priv->dock);
+
   gdk_window_get_origin (priv->dock->window, &dx, &dy);
+  dx += priv->dock->allocation.x;
   dy += priv->dock->allocation.y;
+
   gdk_window_get_origin (priv->scale->window, &sx, &sy);
+  sx += priv->scale->allocation.x;
   sy += priv->scale->allocation.y;
-  ystartoff = sy - dy;
+
   priv->timeout = TRUE;
 
   /* position (needs widget to be shown already) */
   v = gtk_scale_button_get_value (button) / (adj->upper - adj->lower);
-  x += (widget->allocation.width - priv->dock->allocation.width) / 2;
-  y -= ystartoff;
-  y -= GTK_RANGE (priv->scale)->min_slider_size / 2;
-  m = priv->scale->allocation.height -
-      GTK_RANGE (priv->scale)->min_slider_size;
-  y -= m * (1.0 - v);
+
+  if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+    {
+      startoff = sy - dy;
+
+      x += (widget->allocation.width - priv->dock->allocation.width) / 2;
+      y -= startoff;
+      y -= GTK_RANGE (priv->scale)->min_slider_size / 2;
+      m = priv->scale->allocation.height -
+          GTK_RANGE (priv->scale)->min_slider_size;
+      y -= m * (1.0 - v);
+    }
+  else
+    {
+      startoff = sx - dx;
+
+      x -= startoff;
+      y += (widget->allocation.height - priv->dock->allocation.height) / 2;
+      x -= GTK_RANGE (priv->scale)->min_slider_size / 2;
+      m = priv->scale->allocation.width -
+          GTK_RANGE (priv->scale)->min_slider_size;
+      x -= m * v;
+    }
 
   /* Make sure the dock stays inside the monitor */
   if (event->type == GDK_BUTTON_PRESS)
@@ -770,7 +969,11 @@ gtk_scale_popup (GtkWidget *widget,
 						 button_event->y_root);
       gdk_screen_get_monitor_geometry (screen, monitor, &rect);
 
-      y += button_event->y;
+      if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+        y += button_event->y;
+      else
+        x += button_event->x;
+
       if (y < rect.y)
 	y = rect.y;
       else if (y + d->allocation.height > rect.height + rect.y)
@@ -822,11 +1025,23 @@ gtk_scale_popup (GtkWidget *widget,
       /* position: the X position isn't relevant, halfway will work just fine.
        * The vertical position should be *exactly* in the middle of the slider
        * of the scale; if we don't do that correctly, it'll move from its current
-       * position, which means a position change on-click, which is bad. */
-      e->x = priv->scale->allocation.width / 2;
-      m = priv->scale->allocation.height -
-	GTK_RANGE (priv->scale)->min_slider_size;
-      e->y = ((1.0 - v) * m) + GTK_RANGE (priv->scale)->min_slider_size / 2;
+       * position, which means a position change on-click, which is bad.
+       */
+      if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+        {
+          e->x = priv->scale->allocation.width / 2;
+          m = priv->scale->allocation.height -
+              GTK_RANGE (priv->scale)->min_slider_size;
+          e->y = ((1.0 - v) * m) + GTK_RANGE (priv->scale)->min_slider_size / 2;
+        }
+      else
+        {
+          e->y = priv->scale->allocation.height / 2;
+          m = priv->scale->allocation.width -
+              GTK_RANGE (priv->scale)->min_slider_size;
+          e->x = (v * m) + GTK_RANGE (priv->scale)->min_slider_size / 2;
+        }
+
       gtk_widget_event (priv->scale, (GdkEvent *) e);
       e->window = button_event->window;
       gdk_event_free ((GdkEvent *) e);
@@ -847,7 +1062,7 @@ gtk_scale_button_press (GtkWidget      *widget,
 }
 
 static void
-gtk_scale_button_popup_from_bindings (GtkWidget *widget)
+gtk_scale_button_popup (GtkWidget *widget)
 {
   GdkEvent *ev;
 
@@ -912,7 +1127,7 @@ cb_button_timeout (gpointer user_data)
   if (priv->click_id == 0)
     return FALSE;
 
-  adj = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  adj = priv->adjustment;
 
   val = gtk_scale_button_get_value (button);
   val += priv->direction;
@@ -948,7 +1163,7 @@ cb_button_press (GtkWidget      *widget,
 
   button = GTK_SCALE_BUTTON (user_data);
   priv = button->priv;
-  adj = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  adj = priv->adjustment;
 
   if (priv->click_id != 0)
     g_source_remove (priv->click_id);
@@ -1057,7 +1272,7 @@ cb_dock_button_press (GtkWidget      *widget,
 }
 
 static void
-gtk_scale_button_popdown_from_bindings (GtkWidget *widget)
+gtk_scale_button_popdown (GtkWidget *widget)
 {
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
@@ -1084,7 +1299,7 @@ cb_dock_key_release (GtkWidget   *widget,
 {
   if (event->keyval == GDK_Escape)
     {
-      gtk_scale_button_popdown_from_bindings (GTK_WIDGET (user_data));
+      gtk_scale_button_popdown (GTK_WIDGET (user_data));
       return TRUE;
     }
 
@@ -1111,32 +1326,43 @@ cb_scale_grab_notify (GtkWidget *widget,
  * Scale stuff.
  */
 
-#define GTK_TYPE_SCALE_BUTTON_SCALE \
-  (gtk_scale_button_scale_get_type ())
-#define GTK_SCALE_BUTTON_SCALE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_SCALE_BUTTON_SCALE, \
-			       GtkScaleButtonScale))
+#define GTK_TYPE_SCALE_BUTTON_VSCALE    (_gtk_scale_button_vscale_get_type ())
+#define GTK_SCALE_BUTTON_VSCALE(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_SCALE_BUTTON_VSCALE, GtkScaleButtonVScale))
+#define GTK_IS_SCALE_BUTTON_VSCALE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_SCALE_BUTTON_VSCALE))
 
-typedef struct _GtkScaleButtonScaleClass {
-  GtkVScaleClass parent_class;
-} GtkScaleButtonScaleClass;
-
-typedef struct _GtkScaleButtonScale {
+typedef struct _GtkScaleButtonVScale {
   GtkVScale parent;
   GtkScaleButton *button;
-} GtkScaleButtonScale;
+} GtkScaleButtonVScale;
 
-static GType	gtk_scale_button_scale_get_type	  (void);
-static void	gtk_scale_button_scale_class_init (GtkScaleButtonScaleClass *klass);
-static gboolean	gtk_scale_button_scale_press	  (GtkWidget                *widget,
-						   GdkEventButton           *event);
-static gboolean gtk_scale_button_scale_release	  (GtkWidget                *widget,
-						   GdkEventButton           *event);
+typedef struct _GtkScaleButtonVScaleClass {
+  GtkVScaleClass parent_class;
+} GtkScaleButtonVScaleClass;
 
-G_DEFINE_TYPE (GtkScaleButtonScale, gtk_scale_button_scale, GTK_TYPE_VSCALE)
+#define GTK_TYPE_SCALE_BUTTON_HSCALE    (_gtk_scale_button_hscale_get_type ())
+#define GTK_SCALE_BUTTON_HSCALE(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_SCALE_BUTTON_HSCALE, GtkScaleButtonHScale))
+#define GTK_IS_SCALE_BUTTON_HSCALE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_SCALE_BUTTON_HSCALE))
+
+typedef struct _GtkScaleButtonHScale {
+  GtkVScale parent;
+  GtkScaleButton *button;
+} GtkScaleButtonHScale;
+
+typedef struct _GtkScaleButtonHScaleClass {
+  GtkVScaleClass parent_class;
+} GtkScaleButtonHScaleClass;
+
+static gboolean	gtk_scale_button_scale_press   (GtkWidget      *widget,
+                                                GdkEventButton *event);
+static gboolean gtk_scale_button_scale_release (GtkWidget      *widget,
+                                                GdkEventButton *event);
+
+G_DEFINE_TYPE (GtkScaleButtonVScale, _gtk_scale_button_vscale, GTK_TYPE_VSCALE)
+
+G_DEFINE_TYPE (GtkScaleButtonHScale, _gtk_scale_button_hscale, GTK_TYPE_HSCALE)
 
 static void
-gtk_scale_button_scale_class_init (GtkScaleButtonScaleClass *klass)
+_gtk_scale_button_vscale_class_init (GtkScaleButtonVScaleClass *klass)
 {
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
   GtkRangeClass *gtkrange_class = GTK_RANGE_CLASS (klass);
@@ -1147,73 +1373,134 @@ gtk_scale_button_scale_class_init (GtkScaleButtonScaleClass *klass)
 }
 
 static void
-gtk_scale_button_scale_init (GtkScaleButtonScale *scale)
+_gtk_scale_button_hscale_class_init (GtkScaleButtonHScaleClass *klass)
+{
+  GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
+  GtkRangeClass *gtkrange_class = GTK_RANGE_CLASS (klass);
+
+  gtkwidget_class->button_press_event = gtk_scale_button_scale_press;
+  gtkwidget_class->button_release_event = gtk_scale_button_scale_release;
+  gtkrange_class->value_changed = gtk_scale_button_scale_value_changed;
+}
+
+static void
+_gtk_scale_button_vscale_init (GtkScaleButtonVScale *scale)
+{
+}
+
+static void
+_gtk_scale_button_hscale_init (GtkScaleButtonHScale *scale)
 {
 }
 
 static GtkWidget *
-gtk_scale_button_scale_new (GtkScaleButton *button,
-			    gdouble         min,
-			    gdouble         max,
-			    gdouble         step)
+gtk_scale_button_scale_box_new (GtkScaleButton *button)
 {
-  GtkScaleButtonScale *scale;
-  GtkScaleButtonPrivate *priv;
-  GtkObject *adj;
+  GtkScaleButtonPrivate *priv = button->priv;
+  GtkScaleButtonVScale *scale;
+  GtkWidget *box;
 
-  priv = button->priv;
-  scale = g_object_new (GTK_TYPE_SCALE_BUTTON_SCALE, NULL);
-  adj = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
-  gtk_range_set_adjustment (GTK_RANGE (scale), GTK_ADJUSTMENT (adj));
+  if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+    {
+      box = gtk_vbox_new (FALSE, 0);
+
+      scale = g_object_new (GTK_TYPE_SCALE_BUTTON_VSCALE, NULL);
+      priv->scale = GTK_WIDGET (scale);
+
+      gtk_widget_set_size_request (priv->scale, -1, SCALE_SIZE);
+      gtk_range_set_inverted (GTK_RANGE (priv->scale), TRUE);
+
+      gtk_box_pack_start (GTK_BOX (box), button->plus_button, TRUE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (box), priv->scale, TRUE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (box), button->minus_button, TRUE, FALSE, 0);
+    }
+  else
+    {
+      box = gtk_hbox_new (FALSE, 0);
+
+      scale = g_object_new (GTK_TYPE_SCALE_BUTTON_HSCALE, NULL);
+      priv->scale = GTK_WIDGET (scale);
+
+      gtk_widget_set_size_request (priv->scale, SCALE_SIZE, -1);
+
+      gtk_box_pack_start (GTK_BOX (box), button->minus_button, TRUE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (box), priv->scale, TRUE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (box), button->plus_button, TRUE, FALSE, 0);
+    }
+
+  gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
+  gtk_range_set_adjustment (GTK_RANGE (scale),
+                            GTK_ADJUSTMENT (priv->adjustment));
   scale->button = button;
 
-  return GTK_WIDGET (scale);
+  g_signal_connect (priv->scale, "grab-notify",
+                    G_CALLBACK (cb_scale_grab_notify), button);
+
+  /* FIXME: without this, the popup window appears as a square
+   * after changing the orientation
+   */
+  gtk_window_resize (GTK_WINDOW (priv->dock), 1, 1);
+
+  return box;
 }
 
 static gboolean
 gtk_scale_button_scale_press (GtkWidget      *widget,
 			      GdkEventButton *event)
 {
-  GtkScaleButtonScale *scale;
   GtkScaleButtonPrivate *priv;
+  GtkWidgetClass *widget_class;
 
-  scale = GTK_SCALE_BUTTON_SCALE (widget);
-  priv = scale->button->priv;
+  if (GTK_IS_SCALE_BUTTON_VSCALE (widget))
+    {
+      priv = GTK_SCALE_BUTTON_VSCALE (widget)->button->priv;
+      widget_class = GTK_WIDGET_CLASS (_gtk_scale_button_vscale_parent_class);
+    }
+  else
+    {
+      priv = GTK_SCALE_BUTTON_HSCALE (widget)->button->priv;
+      widget_class = GTK_WIDGET_CLASS (_gtk_scale_button_hscale_parent_class);
+    }
 
   /* the scale will grab input; if we have input grabbed, all goes
    * horribly wrong, so let's not do that. */
   gtk_grab_remove (priv->dock);
 
-  return GTK_WIDGET_CLASS (gtk_scale_button_scale_parent_class)->button_press_event (widget, event);
+  return widget_class->button_press_event (widget, event);
 }
 
 static gboolean
 gtk_scale_button_scale_release (GtkWidget      *widget,
 				GdkEventButton *event)
 {
-  GtkScaleButtonScale *scale;
-  GtkScaleButtonPrivate *priv;
+  GtkScaleButton *button;
   GtkWidgetClass *widget_class;
   gboolean res;
 
-  scale = GTK_SCALE_BUTTON_SCALE (widget);
-  priv = scale->button->priv;
+  if (GTK_IS_SCALE_BUTTON_VSCALE (widget))
+    {
+      button = GTK_SCALE_BUTTON_VSCALE (widget)->button;
+      widget_class = GTK_WIDGET_CLASS (_gtk_scale_button_vscale_parent_class);
+    }
+  else
+    {
+      button = GTK_SCALE_BUTTON_HSCALE (widget)->button;
+      widget_class = GTK_WIDGET_CLASS (_gtk_scale_button_hscale_parent_class);
+    }
 
-  widget_class = GTK_WIDGET_CLASS (gtk_scale_button_scale_parent_class);
-
-  if (priv->timeout)
+  if (button->priv->timeout)
     {
       /* if we did a quick click, leave the window open; else, hide it */
-      if (event->time > priv->pop_time + priv->click_timeout)
+      if (event->time > button->priv->pop_time + button->priv->click_timeout)
         {
 
-	  gtk_scale_button_release_grab (scale->button, event);
+	  gtk_scale_button_release_grab (button, event);
 	  widget_class->button_release_event (widget, event);
 
 	  return TRUE;
 	}
 
-      priv->timeout = FALSE;
+      button->priv->timeout = FALSE;
     }
 
   res = widget_class->button_release_event (widget, event);
@@ -1222,8 +1509,9 @@ gtk_scale_button_scale_release (GtkWidget      *widget,
    * it back so we can catch out-of-scale clicks and hide the popup,
    * so I basically want a g_signal_connect_after_always(), but I can't
    * find that, so we do this complex 'first-call-parent-then-do-actual-
-   * action' thingy... */
-  gtk_grab_add (priv->dock);
+   * action' thingy...
+   */
+  gtk_grab_add (button->priv->dock);
 
   return res;
 }
@@ -1247,7 +1535,7 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
 				priv->size);
       return;
     }
- 
+
   num_icons = g_strv_length (priv->icon_list);
 
   /* The 1-icon special case */
@@ -1260,7 +1548,7 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
     }
 
   range = GTK_RANGE (priv->scale);
-  adj = gtk_range_get_adjustment (range);
+  adj = priv->adjustment;
   value = gtk_scale_button_get_value (button);
 
   /* The 2-icons special case */
@@ -1307,10 +1595,14 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
 static void
 gtk_scale_button_scale_value_changed (GtkRange *range)
 {
-  GtkScaleButtonScale *scale = GTK_SCALE_BUTTON_SCALE (range);
-  GtkScaleButton *button = scale->button;
+  GtkScaleButton *button;
   gdouble value;
 
+  if (GTK_IS_SCALE_BUTTON_VSCALE (range))
+    button = GTK_SCALE_BUTTON_VSCALE (range)->button;
+  else
+    button = GTK_SCALE_BUTTON_HSCALE (range)->button;
+
   value = gtk_range_get_value (range);
 
   gtk_scale_button_update_icon (button);
@@ -1318,7 +1610,6 @@ gtk_scale_button_scale_value_changed (GtkRange *range)
   g_signal_emit (button, signals[VALUE_CHANGED], 0, value);
   g_object_notify (G_OBJECT (button), "value");
 }
- 
 
 #define __GTK_SCALE_BUTTON_C__
 #include "gtkaliasdef.c"
